"""YAGO3's ttle file contain escaped unicode symbol like `\u0022`. This file
percentage encoded the ttl files.
"""
import re
import sys
import os

def escape_file(path):
    paths, filename = os.path.split(path)
    with open(path, 'r') as f1, open(os.path.join(paths, "escaped_"+filename), 'w') as f2:
        content = f1.read()
        escaped = re.sub(r'\\u00(\d+)', r'%\1', content)
        escaped = re.sub(r'\\\\', r'%92', escaped)
        f2.write(escaped)

escape_file(sys.argv[1])



"""data utils function"""

import numpy as np

def unpack(triple):
    """unpacks triple into (h, r, t). Can take Triple or TripleIndex."""
    return triple.head, triple.relation, triple.tail

def shuffle_triples(triples):
    """Shuffle triples."""
    return np.random.permutation(triples)

def split_golden_set(triples, valid_ratio, test_ratio):
    """Split the data into train/valid/test sets."""
    assert valid_ratio >= 0.0
    assert test_ratio >= 0.0
    num_valid = int(len(triples) * valid_ratio)
    num_test = int(len(triples) * test_ratio)
    valid_set = triples[:num_valid]
    test_set = triples[num_valid:num_valid+num_test]
    train_set = triples[num_valid+num_test:]
    assert len(valid_set) + len(test_set) + len(train_set) == len(triples)

    return train_set, valid_set, test_set

def _transform_triple_numpy(x):
    """Transform triple index into a 1-D numpy array."""
    return np.array([x.head, x.relation, x.tail], dtype=np.int64)

def pack_triples_numpy(triples):
    """Packs a list of triple indexes into a 2D numpy array."""
    if len(triples) == 0:
        return np.array([], dtype=np.int64)
    return np.stack(list(map(_transform_triple_numpy, triples)), axis=0)



"""Collect dataset statistics."""

from itertools import groupby
import kgekit.io

ONE_TO_ONE_MARGIN_LINE = 1.5
# Check this when modify:
# https://gist.github.com/fantasticfears/f8053480ff1db6da83cb30e446ab2181
def calculate_one_or_many(translation_filename, *triple_filenames):
    _, relations = kgekit.io.relationead_translation(translation_filename)
    relations = dict(relations)
    triples = []
    for filename in triple_filenames:
        tmp, _ = kgekit.io.read_triple_indexes(filename)
        triples += tmp

    triples.sort(key=lambda x: (x.relation, x.head, x.tail))
    grouped = [(k, list(g)) for k, g in groupby(triples, key=lambda x: x.relation)]
    num_of_relations = len(grouped)
    head_per_tail_list = [0] * num_of_relations
    tail_per_head_list = [0] * num_of_relations

    one_to_one = []
    one_to_many = []
    many_to_one = []
    many_to_many = []

    for relation, triples in grouped:
        head_list = []
        tail_list = []
        for triple in triples:
            head_list.append(triple.head)
            tail_list.append(triple.tail)
        head_set = set(head_list)
        tail_set = set(tail_list)
        head_per_tail = len(head_list) / len(tail_set)
        tail_per_head = len(tail_list) / len(head_set)
        head_per_tail_list[relation] = head_per_tail
        tail_per_head_list[relation] = tail_per_head
        if head_per_tail < ONE_TO_ONE_MARGIN_LINE and tail_per_head < ONE_TO_ONE_MARGIN_LINE:
            one_to_one.append(relation)
        elif head_per_tail >= ONE_TO_ONE_MARGIN_LINE and tail_per_head < ONE_TO_ONE_MARGIN_LINE:
            many_to_one.append(relation)
        elif head_per_tail < ONE_TO_ONE_MARGIN_LINE and tail_per_head >= ONE_TO_ONE_MARGIN_LINE:
            one_to_many.append(relation)
        else:
            many_to_many.append(relation)

    return relations, (one_to_one, one_to_many, many_to_one, many_to_many), (head_per_tail_list, tail_per_head_list)




import sys
from collections import defaultdict
import kgekit.io
import kgekit.data
import random


def _get_ent_rels(triples):
    entities_relation = defaultdict(set)
    for triple in triples:
        h, r, t = kgekit.data.unpack(triple)
        entities_relation[h].add((t,r))
        entities_relation[t].add((h,r))
    return entities_relation

def _get_deficit_ents(triples):
    ent_rels = _get_ent_rels(triples)
    deficit_entities = []
    for k, v in ent_rels.items():
        deficit_entities.append(k) if len(v) < 10 else next
    return frozenset(deficit_entities)


def validate_10_relations(triples, getEntityFromId):
    entities_relation = _get_ent_rels(triples)
    deficit_entities = []
    for k, v in entities_relation.items():
        deficit_entities.append(getEntityFromId(k)) if len(v) < 10 else next
    num_deficits = len(deficit_entities)
    if num_deficits > 0:
        print(deficit_entities)
        print("------")
        print(entities_relation)
        raise RuntimeError(str(num_deficits) + " deficit entities found. Validation failed on some entities without sufficient relations")

def _get_reverse_triples(triple_indexes):
    mapping = defaultdict(list)
    for triple in triple_indexes:
        h, r, t = kgekit.data.unpack(triple)
        entity_pair = frozenset([h,t])
        mapping[entity_pair].append(triple)
    reverse_triples = []
    for k,v in mapping.items():
        reverse_triples.append(v) if len(v) > 1 else next
    return reverse_triples

def validate_reverse(indexes, getEntityFromId, getRelationFromId):
    reverse_triples = _get_reverse_triples(indexes)
    if len(reverse_triples) > 0:
        print(reverse_triples)
        raise RuntimeError(str(len(reverse_triples)) + " reverse triples found.")

def validate(triples):
    indexer = kgekit.EntityNumberIndexer(triples, "hrt")
    indexes = indexer.indexes()
    validate_10_relations(indexes, indexer.getEntityFromId)
    validate_reverse(indexes, indexer.getEntityFromId, indexer.getRelationFromId)

def _write_triple(out_filename, new_set, indexer, seperator):
    with open(out_filename, 'w') as f:
        for idx in new_set:
            f.write(indexer.getEntityFromId(idx.head) + seperator + indexer.getRelationFromId(idx.relation) + seperator + indexer.getEntityFromId(idx.tail) + "\n")

def remove10(filename, out_filename):
    seperator = " "
    triples, num_failed = kgekit.io.read_triples(filename, "hrt", seperator)
    if num_failed > 0:
        input("Failed reading " + str(num_failed) + " triple(s). Press Enter to continue...")
    indexer = kgekit.EntityNumberIndexer(triples, "hrt")
    indexes = indexer.indexes()
    ents = _get_deficit_ents(indexes)
    removed_ents = len(ents)
    original = set(indexes)
    to_deleted = set()
    for triple in indexes:
        h, r, t = kgekit.data.unpack(triple)
        if h in ents or t in ents:
            to_deleted.add(triple)
    new_set = original - to_deleted
    removed_triples = len(original) - len(new_set)

    print("Removed entities " + str(removed_ents))
    print("Removed triples " + str(removed_triples))
    _write_triple(out_filename, new_set, indexer, seperator)


def remove_reverse(filename, out_filename):
    seperator = " "
    triples, num_failed = kgekit.io.read_triples(filename, "hrt", seperator)
    if num_failed > 0:
        input("Failed reading " + str(num_failed) + " triple(s). Press Enter to continue...")
    indexer = kgekit.EntityNumberIndexer(triples, "hrt")
    indexes = indexer.indexes()
    original = frozenset(indexes)
    reverse_triples = _get_reverse_triples(indexes)

    # calculate frequences of all conflicted triples
    freqs = defaultdict(int)
    for reverses in reverse_triples:
        # reverse is a list of conflicting triple of interest
        for r in reverses:
            freqs[r.relation] += 1
    print(freqs)

    to_deleted = set()
    # choose one to remove
    for reverses in reverse_triples:
        sampled = random.sample(reverses, 1)
        to_deleted.add(sampled[0])

    new_set = original - to_deleted
    removed_reverse = len(original) - len(new_set)
    print("Removed reverse triples " + str(removed_reverse))
    _write_triple(out_filename, new_set, indexer, seperator)


if __name__ == '__main__':
    if 'validate' == sys.argv[1]:
        triples, num_failed = kgekit.io.read_triples(sys.argv[2], "hrt", " ")
        if num_failed > 0:
            input("Failed reading " + str(num_failed) + " triple(s). Press Enter to continue...")
        validate(triples)
    elif 'remove_10' == sys.argv[1]:
        remove10(sys.argv[2], sys.argv[3])
    elif 'remove_reverse' == sys.argv[1]:
        remove_reverse(sys.argv[2], sys.argv[3])



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: triple_index.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='triple_index.proto',
  package='',
  syntax='proto3',
  serialized_pb=_b('\n\x12triple_index.proto\"\xce\x01\n\x0bTranslation\x12,\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x1a.Translation.EntitiesEntry\x12.\n\trelations\x18\x02 \x03(\x0b\x32\x1b.Translation.RelationsEntry\x1a/\n\rEntitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x1a\x30\n\x0eRelationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x62\x06proto3')
)




_TRANSLATION_ENTITIESENTRY = _descriptor.Descriptor(
  name='EntitiesEntry',
  full_name='Translation.EntitiesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Translation.EntitiesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Translation.EntitiesEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=132,
  serialized_end=179,
)

_TRANSLATION_RELATIONSENTRY = _descriptor.Descriptor(
  name='RelationsEntry',
  full_name='Translation.RelationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Translation.RelationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='Translation.RelationsEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=181,
  serialized_end=229,
)

_TRANSLATION = _descriptor.Descriptor(
  name='Translation',
  full_name='Translation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entities', full_name='Translation.entities', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relations', full_name='Translation.relations', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_TRANSLATION_ENTITIESENTRY, _TRANSLATION_RELATIONSENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=23,
  serialized_end=229,
)

_TRANSLATION_ENTITIESENTRY.containing_type = _TRANSLATION
_TRANSLATION_RELATIONSENTRY.containing_type = _TRANSLATION
_TRANSLATION.fields_by_name['entities'].message_type = _TRANSLATION_ENTITIESENTRY
_TRANSLATION.fields_by_name['relations'].message_type = _TRANSLATION_RELATIONSENTRY
DESCRIPTOR.message_types_by_name['Translation'] = _TRANSLATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Translation = _reflection.GeneratedProtocolMessageType('Translation', (_message.Message,), dict(

  EntitiesEntry = _reflection.GeneratedProtocolMessageType('EntitiesEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSLATION_ENTITIESENTRY,
    __module__ = 'triple_index_pb2'
    # @@protoc_insertion_point(class_scope:Translation.EntitiesEntry)
    ))
  ,

  RelationsEntry = _reflection.GeneratedProtocolMessageType('RelationsEntry', (_message.Message,), dict(
    DESCRIPTOR = _TRANSLATION_RELATIONSENTRY,
    __module__ = 'triple_index_pb2'
    # @@protoc_insertion_point(class_scope:Translation.RelationsEntry)
    ))
  ,
  DESCRIPTOR = _TRANSLATION,
  __module__ = 'triple_index_pb2'
  # @@protoc_insertion_point(class_scope:Translation)
  ))
_sym_db.RegisterMessage(Translation)
_sym_db.RegisterMessage(Translation.EntitiesEntry)
_sym_db.RegisterMessage(Translation.RelationsEntry)


_TRANSLATION_ENTITIESENTRY.has_options = True
_TRANSLATION_ENTITIESENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_TRANSLATION_RELATIONSENTRY.has_options = True
_TRANSLATION_RELATIONSENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)



import sys

def strip_labels(filename):
    """Strips labels."""
    labels = []
    with open(filename) as f, open('processed_labels.txt', 'w') as f1:
        for l in f:
            if l.startswith('#'):
                next
            l = l.replace(" .", '')
            l = l.replace(">\tskos:prefLabel\t", ' ')
            l = l.replace("<", '')
            l = l.replace(">\trdfs:label\t", ' ')
            f1.write(l)

if __name__ == '__main__':
    strip_labels(sys.argv[1])



import os
import re
import sys
import glob
import sysconfig
import platform
import subprocess
import kgekit

from distutils.version import LooseVersion
from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext
from shutil import copyfile, copymode
from setuptools import find_packages, setup, Command


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: " +
                ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)',
                                         out.decode()).group(1))
            if cmake_version < '3.1.0':
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def copy_test_file(self, ext, src_file):

        '''
        Copy ``src_file`` to `tests/bin` directory, ensuring parent directory
        exists. Messages like `creating directory /path/to/package` and
        `copying directory /src/path/to/package -> path/to/package` are
        displayed on standard output. Adapted from scikit-build.
        '''
        # Create directory if needed
        dest_dir = os.path.join(os.path.dirname(
            os.path.abspath(__file__)), ext.name, 'tests', 'bin')
        if dest_dir != "" and not os.path.exists(dest_dir):
            print("creating directory {}".format(dest_dir))
            os.makedirs(dest_dir)

        # Copy file
        dest_file = os.path.join(dest_dir, os.path.basename(src_file))
        print("copying {} -> {}".format(src_file, dest_file))
        copyfile(src_file, dest_file)
        copymode(src_file, dest_file)

    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir,
                      '-DPYTHON_EXECUTABLE=' + sys.executable,
                      '-DCMAKE_DEBUG_POSTFIX=""']

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(
                cfg.upper(),
                extdir)]
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j2']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''),
            self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args,
                              cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args,
                              cwd=self.build_temp)
        # Copy *_test file to tests directory
        test_bin = os.path.join(self.build_temp, 'kgedata', 'kgedata_test')
        self.copy_test_file(ext, test_bin)
        print()  # Add an empty line for cleaner output


class CleanCommand(Command):
    """Custom clean command to tidy up the project root."""
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        os.system('rm -vrf ./build ./dist ./*.pyc ./*.tgz ./*.egg-info')


setup(name='kgekit',
      version=kgekit.__version__,
      description='knowledge representation tools',
      long_description=open("README.md", "r").read(),
      long_description_content_type="text/markdown",
      url='http://github.com/fantasticfears/kgekit',
      author='Erick Guan',
      author_email='fantasticfears@gmail.com',
      license='GPLv2',
      packages=find_packages(),
      install_requires=[
        'protobuf>=3',
        'numpy>=1.10'
      ],
      cmdclass=dict(build_ext=CMakeBuild, clean=CleanCommand),
      ext_modules=[CMakeExtension('kgekit')],
      test_suite='tests',
      zip_safe=False,
      classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
      ],
)



"""Shuffle and split the data into train/valid/test sets."""

import sys
import kgekit.data
import kgekit.io

def shuffled_split(filename):
    triples, num_failed = kgekit.io.read_triples(filename, 'hrt', ' ')
    if num_failed > 0:
        print("There is " + str(num_failed) + " failures.")
    indexer = kgekit.EntityNumberIndexer(triples, "hrt")
    indexes = indexer.indexes()
    kgekit.io.write_index_translation('translation.protobuf', indexer.entityIdMap(), indexer.relationIdMap())
    indexes = kgekit.data.shuffle_triples(indexes)
    train, valid, test = kgekit.data.split_golden_set(indexes, 0.005, 0.005)
    kgekit.io.write_triples('train.txt', train)
    kgekit.io.write_triples('valid.txt', valid)
    kgekit.io.write_triples('test.txt', test)

if __name__ == '__main__':
    shuffled_split(sys.argv[1])
